#!/bin/bash

if [ $# = 0 ]; then
	echo "USAGE: all.sh config_file [sge_param]"
	exit 1
fi


# load configuration
conf=$1
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

if [ ! -x $conf ] ; then
	echo "ERROR: all.sh config file not found - $conf"
	exit 1
fi
if [ ! -x $conf_global ] ; then
	echo "ERROR: all.sh global config file not found - $conf_global"
	exit 1
fi

source $conf_global
source $conf

# append full path to the configuration file name if one does not exist (for SGE)
if [ ${conf:0:1} != "/" ] && [ ${conf:0:1} != "." ] ; then
	conf="`pwd`/$conf"
	conf_global="`pwd`/$conf_global"
fi

# run this script again over SGE if it is enabled
if [ $use_sge == 1 ] && [ "$2" != "sge" ] && [ "$3" != "sge" ] ; then
	echo "Submitting script ${script_dir}/all.sh $conf to SGE..."
	sge_params=""
	if [ "$2" != "sge" ] ; then
		sge_params="$2"
	else
		sge_params="$3"
	fi
	${script_dir}/manage_task.sh $conf "${script_dir}/all.sh $conf sge" "std" $sge_params $global_sge_params
	exit 0
fi

# split labels into more shorter labels (states) --------------------------------------------------------
if ( [ "$num_states" != "" ] && [ "$num_states" != "1" ] ) || [ "$num_states_file" != "" ] ; then

	echo "Splitting labels into states..."
	
	new_out_dir="$out_dir/it1"
	rm -r -f $new_out_dir
	mkdir -p $out_dir/it1/scripts
	nice -n 19 cp $conf_global $new_out_dir/scripts

	mkdir -p $new_out_dir/source/pfiles
	mkdir -p $new_out_dir/source/dicts
	nice -n 19 cp -r $pfile_dir/mlfs $new_out_dir/source/mlfs
	nice -n 19 cp -r $pfile_dir/lists $new_out_dir/source/lists

	if [ "$num_states_file" != "" ] ; then
		if [ ! -e $new_out_dir/source/pfiles/train.pfile ] ; then
			${bin_dir}/pfile_labs -i $pfile_dir/pfiles/train.pfile -o $new_out_dir/source/pfiles/train.pfile -sn $num_states_file
		fi
		if [ ! -e $new_out_dir/source/pfiles/test.pfile ] ; then
			${bin_dir}/pfile_labs -i $pfile_dir/pfiles/test.pfile -o $new_out_dir/source/pfiles/test.pfile -sn $num_states_file
		fi
		if [ ! -e $new_out_dir/source/pfiles/cv.pfile ] ; then
			${bin_dir}/pfile_labs -i $pfile_dir/pfiles/cv.pfile -o $new_out_dir/source/pfiles/cv.pfile -sn $num_states_file
		fi
		if [ ! -e $new_out_dir/source/dicts/dict ] ; then
			${bin_dir}/pfile_dict -rd $pfile_dir/dicts/dict -sd $new_out_dir/source/dicts/dict -sn $num_states_file
		fi
	else
		if [ ! -e $new_out_dir/source/pfiles/train.pfile ] ; then
			${bin_dir}/pfile_labs -i $pfile_dir/pfiles/train.pfile -o $new_out_dir/source/pfiles/train.pfile -st $num_states
		fi
		if [ ! -e $new_out_dir/source/pfiles/test.pfile ] ; then
			${bin_dir}/pfile_labs -i $pfile_dir/pfiles/test.pfile -o $new_out_dir/source/pfiles/test.pfile -st $num_states
		fi
		if [ ! -e $new_out_dir/source/pfiles/cv.pfile ] ; then
			${bin_dir}/pfile_labs -i $pfile_dir/pfiles/cv.pfile -o $new_out_dir/source/pfiles/cv.pfile -st $num_states
		fi
		if [ ! -e $new_out_dir/source/dicts/dict ] ; then
			${bin_dir}/pfile_dict -rd $pfile_dir/dicts/dict -sd $new_out_dir/source/dicts/dict -st $num_states
		fi
	fi

	new_num_classes=`wc -l $new_out_dir/source/dicts/dict | awk '{print $1}'`

	awk '/^exp_dir=/     {print gensub("^exp_dir=\"(.+)\"$", "exp_dir=\"\\1/it1\"", "");}
	     /^num_classes=/ {print gensub("^num_classes=(.+)$", "num_classes='$new_num_classes'", "");}
	     /^pfile_dir=/   {print gensub("^pfile_dir=(.+)$", "pfile_dir='$new_out_dir/source'", "");}
	     !/^exp_dir=/ && !/^num_classes=/ && !/^pfile_dir=/ && !/^num_states=/ && !/^num_states_file=/ {print;}' $conf > $new_out_dir/scripts/run.sh
	     
	chmod ug+x $new_out_dir/scripts/run.sh
	echo ${script_dir}/all.sh $new_out_dir/scripts/run.sh
	${script_dir}/all.sh $new_out_dir/scripts/run.sh $global_sge_params
	exit 0
fi

# realign labels for a new iteration --------------------------------------------------------------------
# iteraction 1 is without realignment
if [ "$realign_labels" = "1" ] && [ "$max_iterations" != "0" ] &&
   [ "$max_iterations" != "1" ] && [ "$max_iterations" != "" ] ; then

	echo "Realingning labels..."

	# source pfiles, labels contained in will be realigned
	source_pfile_train=$source_pfile_dir/pfiles/train.pfile
	source_pfile_test=$source_pfile_dir/pfiles/test.pfile
	source_pfile_cv=$source_pfile_dir/pfiles/cv.pfile

	if [ ! -e $source_pfile_train ] || [ ! -e $source_pfile_test ] || [ ! -e $source_pfile_cv ] ; then
		echo "ERROR - one of source pfiles does not exist, check the 'source_pfile_dir' variable!"
		exit 1
	fi

	# source files of posteriors, these posteriors are used in the Forward/Backward algorithm
	# for the state occupation porobability computation
	source_ffout_train=$source_ffout_dir/train.pfile
	source_ffout_test=$source_ffout_dir/test.pfile
	source_ffout_cv=$source_ffout_dir/cv.pfile

	if [ ! -e $source_ffout_train ] || [ ! -e $source_ffout_test ] || [ ! -e $source_ffout_cv ] ; then
		echo "ERROR - one of source PFILES with posteriors does not exist, check the 'source_ffout_dir' variable!"
		exit 1
	fi

	# allocate temporal directory
	work_dir=`${script_dir}/misc/get_workdir.sh $conf`
	echo "Working directory for realignment (all.sh): $work_dir"
	#echo $aa | awk '/^.+\it.+$/ {print}'

	# convert pfiles with posteriors into pfiles
	mkdir -p $out_dir/source/pfiles
	
	# realign labels and create new pfiles of mel-banks (with new labels)
        echo "TRAIN"
	if [ ! -e $out_dir/source/pfiles/train.pfile ] ; then
		${bin_dir}/pfile_realign -i $source_pfile_train -o $out_dir/source/pfiles/train.pfile -p $source_ffout_train
	fi
	echo "TEST"
	if [ ! -e $out_dir/source/pfiles/test.pfile ] ; then
		${bin_dir}/pfile_realign -i $source_pfile_test -o $out_dir/source/pfiles/test.pfile -p $source_ffout_test
	fi
	echo "CV"
	if [ ! -e $out_dir/source/pfiles/cv.pfile ] ; then
		${bin_dir}/pfile_realign -i $source_pfile_cv -o $out_dir/source/pfiles/cv.pfile -p $source_ffout_cv
	fi

	# remove pfiles of posteriors, we do not need them
	rm -f $post_dir/train_post.pfile
	rm -f $post_dir/test_post.pfile
	rm -f $post_dir/cv_post.pfile

	# remove the temporal directory
	${script_dir}/misc/rm_workdir.sh $conf $work_dir

	# copy all neccesary staff into the directory of realigned pfiles
	nice -n 19 cp -r $source_pfile_dir/mlfs $out_dir/source/mlfs
	nice -n 19 cp -r $source_pfile_dir/lists $out_dir/source/lists
	nice -n 19 cp -r $source_pfile_dir/dicts $out_dir/source/dicts
	nice -n 19 cp $source_pfile_dir/configs $out_dir/source
	nice -n 19 cp $source_pfile_dir/hcopy.cfg $out_dir/source

	if [ -e "$source_pfile_dir/test_states.mlf" ] ; then
		nice -n 19 cp $source_pfile_dir/test_states.mlf $out_dir/source
	else
		source_dir=${source_ffout_dir%/*}
		if [ -e $source_dir/rec_m.mlf ] ; then
			${script_dir}/misc/extractMLFLevel.pl $source_dir/rec_m.mlf 0 | sed 's/\.rec/\.lab/' > $out_dir/source/test_states.mlf
		fi
	fi

	# create new script
	mkdir -p $out_dir/scripts
	awk '/^realign_labels=/ {print gensub("^realign_labels=(.+)$", "realign_labels=0", "");}
	     /^pfile_dir=/      {print gensub("^pfile_dir=(.+)$", "pfile_dir='$out_dir/source'", "");}
	     !/^act_iteration=/ && !/^pfile_dir=/ && !/^realign_labels=/ {print;}' $conf > $out_dir/scripts/run1.sh

	chmod ug+x $out_dir/scripts/run1.sh
	${script_dir}/all.sh $out_dir/scripts/run1.sh sge  # run without sge
	exit 0
fi


# work directory ----------------------------------------------------------------------------------------
work_dir=`${script_dir}/misc/get_workdir.sh $conf`
echo "Working directory for all.sh: $work_dir"
machine=`uname -n`
echo "Computer:                     $machine"
time=`date`
echo "Time:                         $time"


# estimate GMM models for VTLN --------------------------------------------------------------------------
if [ "$do_tr_vtln" = "1" ] || [ "$do_te_vtln" = "1" ] || [ "$do_estim_gmm" = "1" ] ; then
	echo "Estimating GMM models for VTLN..."
        mkdir -p $work_dir/vtln/pfiles
        mkdir -p $out_dir/vtln/gmm
	if [ ! -e $out_dir/vtln/gmm/models ] ; then
		if [ ! -e $work_dir/vtln/pfiles/train_par.pfile ] ; then
        		${bin_dir}/$vtln_param -i $train_tnn -o $work_dir/vtln/pfiles/train_par.pfile
		fi
        	last=$(( $tnn_train_sents - 1 ))
        	${bin_dir}/pfile_rcov -r 0:$last -i $work_dir/vtln/pfiles/train_par.pfile -o $out_dir/vtln/gmm/models
        	rm -f $work_dir/vtln/pfiles/train_par.pfile
	fi
fi

# VTLN on train set -------------------------------------------------------------------------------------
if [ "$do_tr_vtln" = "1" ] ; then

	echo "VTLN on the train part..."

	if [ ! -e $out_dir/source_vtln/pfiles/train.pfile ] || [ ! -e $out_dir/source_vtln/pfiles/cv.pfile ] ; then

		# create waveform - mel banks list
		if [ ! -e $pfile_dir/configs ] ; then
			echo "ERROR: file '$pfile_dir/configs' not found!"
			exit 1
		fi

		data_dir_train=`grep data_dir_train $pfile_dir/configs | tr '=' ' ' | awk '{print $2}'`
		data_dir_cv=`grep data_dir_cv $pfile_dir/configs | tr '=' ' ' | awk '{print $2}'`
		data_file_suffix=`grep data_file_suffix $pfile_dir/configs | tr '=' ' ' | awk '{print $2}'`

		mkdir -p $work_dir/vtln
		mkdir -p $out_dir/vtln/features/

		find $data_dir_train -name "*.$data_file_suffix" > $work_dir/vtln/train_dir_files.scp
		find $data_dir_cv -name "*.$data_file_suffix" > $work_dir/vtln/cv_dir_files.scp

		awk '{print $1 ".'$data_file_suffix' '$out_dir/vtln/features/'" $1 ".mb"} ' $train_list > $work_dir/vtln/train_wav_mb1.scp
		awk '{print $1 ".'$data_file_suffix' '$out_dir/vtln/features/'" $1 ".mb"} ' $cv_list > $work_dir/vtln/cv_wav_mb1.scp

		${script_dir}/misc/expand_paths.pl $work_dir/vtln/train_wav_mb1.scp $work_dir/vtln/train_dir_files.scp > $work_dir/vtln/train_wav_mb.scp
		${script_dir}/misc/expand_paths.pl $work_dir/vtln/cv_wav_mb1.scp $work_dir/vtln/cv_dir_files.scp > $work_dir/vtln/cv_wav_mb.scp

		# save labels
		${bin_dir}/pfile_labs -d $dict -i $train_tnn -f $train_list -sl $out_dir/vtln/train_ref.mlf -l "*/"
		${bin_dir}/pfile_labs -d $dict -i $cv_tnn -f $cv_list -sl $out_dir/vtln/cv_ref.mlf -l "*/"

		# vtln
		echo "   Searching for the best warping factors..."
		mkdir -p $out_dir/vtln/log
		${script_dir}/pvtln.pl ${script_dir} $vtln_ncpu $conf $out_dir/vtln $work_dir/vtln/train_wav_mb.scp $pfile_dir/hcopy.cfg $out_dir/vtln/train_ref.mlf $out_dir/vtln/train.log $out_dir/vtln/log
		# - the cv set is an subset of train set
		# ${script_dir}/pvtln.pl ${script_dir} $vtln_ncpu $conf $out_dir/vtln $work_dir/vtln/cv_wav_mb.scp $pfile_dir/hcopy.cfg $out_dir/vtln/cv_ref.mlf $out_dir/vtln/cv.log $out_dir/vtln/log

		awk '{print $2}' $work_dir/vtln/train_wav_mb.scp > $work_dir/vtln/train_mb.scp
		awk '{print $2}' $work_dir/vtln/cv_wav_mb.scp > $work_dir/vtln/cv_mb.scp

		echo "   Generating pfile ..."
		mkdir -p $work_dir/vtln/pfiles
		${bin_dir}/feacat -i $work_dir/vtln/train_mb.scp -l -ip htk -o $work_dir/vtln/pfiles/train_nolab.pfile
		${bin_dir}/feacat -i $work_dir/vtln/cv_mb.scp -l -ip htk -o $work_dir/vtln/pfiles/cv_nolab.pfile
		find $out_dir/vtln/features -name "*.mb" -exec rm {} \;

		echo "   Adding reference labels ..."
		${bin_dir}/pfile_labs -i $work_dir/vtln/pfiles/train_nolab.pfile -o $work_dir/vtln/pfiles/train.pfile -f $train_list -d $pfile_dir/dicts/dict -al $out_dir/vtln/train_ref.mlf
		${bin_dir}/pfile_labs -i $work_dir/vtln/pfiles/cv_nolab.pfile -o $work_dir/vtln/pfiles/cv.pfile -f $cv_list -d $pfile_dir/dicts/dict -al $out_dir/vtln/cv_ref.mlf

		rm -f $work_dir/vtln/pfiles/train_nolab.pfile
		rm -f $work_dir/vtln/pfiles/cv_nolab.pfile

		# prepare new pfile structure
		echo "   Preparing new pfile structure..."

		# source pfiles, labels contained in will be realigned
		if [ ! -d $source_pfile_dir ] ; then
			source_pfile_dir=pfile_dir
		fi

		source_pfile_train=$work_dir/vtln/pfiles/train.pfile
		source_pfile_test=$source_pfile_dir/pfiles/test.pfile
		source_pfile_cv=$work_dir/vtln/pfiles/cv.pfile

		if [ ! -e $source_pfile_train ] || [ ! -e $source_pfile_test ] || [ ! -e $source_pfile_cv ] ; then
			echo "ERROR - one of source pfiles does not exist, VTLN crashed!"
			exit 1
		fi

		# copy all neccesary staff into the directory of realigned pfiles
		mkdir -p $out_dir/source_vtln/pfiles

		if [ -e "$source_pfile_dir/test_states.mlf" ] ; then
			nice -n 19 cp $source_pfile_dir/test_states.mlf $out_dir/source_vtln
		else
			source_dir=${source_ffout_dir%/*}
			if [ -e $source_dir/rec_m.mlf ] ; then
				${script_dir}/misc/extractMLFLevel.pl $source_dir/rec_m.mlf 0 | sed 's/\.rec/\.lab/' > $out_dir/source_vtln/test_states.mlf
			fi
		fi

		nice -n 19 cp -r $source_pfile_dir/mlfs $out_dir/source_vtln/mlfs
		nice -n 19 cp -r $source_pfile_dir/lists $out_dir/source_vtln/lists
		nice -n 19 cp -r $source_pfile_dir/dicts $out_dir/source_vtln/dicts
		nice -n 19 cp $source_pfile_dir/configs $out_dir/source_vtln
		nice -n 19 cp $source_pfile_dir/hcopy.cfg $out_dir/source_vtln
		nice -n 19 mv $source_pfile_train $out_dir/source_vtln/pfiles
		nice -n 19 mv $source_pfile_cv $out_dir/source_vtln/pfiles
	fi

	if [ ! -e $out_dir/source_vtln/pfiles/test.pfile ] ; then
		nice -n 19 cp $source_pfile_test $out_dir/source_vtln/pfiles
	fi

	# create new script
	mkdir -p $out_dir/scripts
	awk '/^realign_labels=/ {print gensub("^realign_labels=(.+)$", "realign_labels=0", "");}
	     /^pfile_dir=/      {print gensub("^pfile_dir=(.+)$", "pfile_dir='$out_dir/source_vtln'", "");}
	     /^do_tr_vtln=/     {print gensub("^do_tr_vtln=(.+)$", "do_tr_vtln=0", "");}
	     !/^act_iteration=/ && !/^pfile_dir=/ && !/^realign_labels=/ && !/^do_tr_vtln=/ {print;}' $conf > $out_dir/scripts/run2.sh

	chmod ug+x $out_dir/scripts/run2.sh
	${script_dir}/all.sh $out_dir/scripts/run2.sh sge $global_sge_params # run without sge
	exit 0
fi


# system ------------------------------------------------------------------------------------------------
if [ "$system" = "STREAMS" ] ; then
	echo "Entering the STREAMS system..."

	# number of streams
	nstreams=1
	while [ "${mfccer[$nstreams]}" != "" ] ; do
		nstreams=$(( $nstreams+1 ))
	done
	nstreams=$(( $nstreams - 1 ))

	if [ $do_train == 1 ] ; then
		echo "   Training ..."

		if [ ! -e $out_dir/weights/${system}_merger.weights ] ; then

			if [ ! -e $work_dir/pfiles/train_mnn_postpr.pfile ] && [ -e $out_dir/pfiles/train_mnn.pfile ] ; then
				mkdir -p $work_dir/pfiles
				nice -n 19 cp $out_dir/pfiles/train_mnn.pfile $work_dir/pfiles
				train_mnn="$work_dir/pfiles/train_mnn.pfile"
				${script_dir}/streams_postprocess_mergerd_streams.sh $conf $train_mnn $work_dir/pfiles/train_mnn_postpr.pfile
			fi

			if [ ! -e $work_dir/pfiles/train_mnn_postpr.pfile ] ; then

				# training - processing of streams
				if [ $do_tr_bands == 1 ] ; then
					echo "      - training streams..."
					if [ $use_sge == 1 ] && [ $nstreams != 1 ] ; then
						nstreams_1=$(( $nstreams - 1 ))
						jid=`${script_dir}/manage_task.sh $conf "${script_dir}/streams_train_stream.sh $conf \\\$SGE_TASK_ID" train_streams "-t 1-$nstreams_1 -jid -m a" | tail -n 1`

						# run one stream on this computer
						${script_dir}/streams_train_stream.sh $conf $nstreams

						${script_dir}/manage_task.sh $conf -waitfor $jid
					else
						for (( i=1 ; i <= $nstreams ; i=i+1 )) ; do
                                			${script_dir}/streams_train_stream.sh $conf $i
						done
					fi
				fi

				# reference labels for merger - training pfile
				mkdir -p $work_dir/labels
				${bin_dir}/pfile_print -i $train_tnn -ns -fr nil -q > $work_dir/labels/merger_train.labs

				# merge rabbin files into one pfile
				echo "      - merging streams..."
				mkdir -p $work_dir/pfiles
				train_mnn="$work_dir/pfiles/train_mnn.pfile"
				rm -f $train_mnn
				${script_dir}/streams_merge_streams.sh $conf $nstreams $out_dir/ffout/train_s "_postpr.pfile" $train_mnn $work_dir/labels/merger_train.labs
				if [ ! -e $train_mnn ] ; then
					echo "ERROR: A PFILE with posteriors does not exist" > /dev/stderr
					exit
				fi
				rm -f $work_dir/labels/merger_train.labs
				rmdir $work_dir/labels

				if [ $keep_train_band_posteriors != 1 ] ; then
					for (( i=0 ; i < $nstreams ; i=i+1 )) ; do
						rm -f "$out_dir/ffout/train_s$i.pfile"
						rm -f "$out_dir/ffout/train_s${i}_postpr.pfile"
					done
					if [ "`ls $out_dir/ffout`" == "" ] ; then
						rmdir $out_dir/ffout;
					fi
				fi

				# postprocessing of merged posteriors and merger training
				${script_dir}/streams_postprocess_mergerd_streams.sh $conf $train_mnn $work_dir/pfiles/train_mnn_postpr.pfile
			fi

			# train merger
			echo "      - training merger..."
			${script_dir}/train_MN.sh $conf ${system}_merger $work_dir/pfiles/train_mnn_postpr.pfile
		fi

		rm -f $work_dir/pfiles/train_mnn.pfile
		rm -f $work_dir/pfiles/train_mnn_postpr.pfile
	fi

	if [ $do_train_fwd == 1 ] ; then
		echo "Froward pass of training set..."

		if [ ! -e $out_dir/ffout/train.pfile ] && [ ! -e $work_dir/ffout/train.pfile ]; then
			mkdir -p $out_dir/ffout
			mkdir -p $work_dir/ffout
			${script_dir}/streams_forward_pass.sh $conf train $train_tnn $work_dir/ffout/train.pfile

			if [ $keep_train_merged_band_posteriors == 1 ] ; then
				mkdir -p $out_dir/pfiles
				if [ ! $work_dir/pfiles/train_mnn.pfile -ef $out_dir/pfiles/train_mnn.pfile ] ; then
					rm -f $out_dir/pfiles/train_mnn.pfile
					mv $work_dir/pfiles/train_mnn.pfile $out_dir/pfiles/train_mnn.pfile
				fi
			else
				rm -f $work_dir/pfiles/train_mnn.pfile
			fi

			rm -f $work_dir/pfiles/train_mnn_postpr.pfile
		fi
	fi

	if [ $do_test == 1 ] ; then
		echo "Froward pass of test set..."

		if [ ! -e $out_dir/ffout/test.pfile ] && [ ! -e $work_dir/ffout/test.pfile ] ; then
			mkdir -p $out_dir/ffout
			mkdir -p $work_dir/ffout
			${script_dir}/streams_forward_pass.sh $conf test $test_tnn $work_dir/ffout/test.pfile

			if [ $keep_test_merged_band_posteriors == 1 ] ; then
				mkdir -p $out_dir/pfiles
				if [ ! $work_dir/pfiles/test_mnn.pfile -ef $out_dir/pfiles/test_mnn.pfile ] ; then
					rm -f $out_dir/pfiles/test_mnn.pfile
					mv $work_dir/pfiles/test_mnn.pfile $out_dir/pfiles/test_mnn.pfile
				fi
			else
				rm -f $work_dir/pfiles/test_mnn.pfile
			fi

			rm -f $work_dir/pfiles/test_mnn_postpr.pfile
		fi
	fi

	if [ $do_cv == 1 ] ; then
		echo "Froward pass of cv set..."

		if [ ! -e $out_dir/ffout/cv.pfile ] && [ ! -e $work_dir/ffout/cv.pfile ] ; then
			mkdir -p $out_dir/ffout
			mkdir -p $work_dir/ffout
			${script_dir}/streams_forward_pass.sh $conf cv $cv_tnn $work_dir/ffout/cv.pfile

			if [ $keep_cv_merged_band_posteriors == 1 ] ; then
				mkdir -p $out_dir/pfiles
				if [ ! $work_dir/pfiles/cv_mnn.pfile -ef $out_dir/pfiles/cv_mnn.pfile ] ; then
					rm -f $out_dir/pfiles/cv_mnn.pfile
					mv $work_dir/pfiles/cv_mnn.pfile $out_dir/pfiles/cv_mnn.pfile
				fi
			else
				rm -f $work_dir/pfiles/cv_mnn.pfile
			fi
		fi
	fi

	if [ "$do_te_vtln" = "1" ] ; then
		echo " VTLN on the test part..."

		if [ "$do_te_vtln_search" = "1" ] ; then

			if [ ! -e $out_dir/vtln/pfiles/test.pfile ] ; then
				mkdir -p $out_dir/vtln/pfiles
				mkdir -p $out_dir/vtln

				# extract state transcription from the MLF
				if [ ! -e $pfile_dir/test_states.mlf ] ; then

					# generate state level MLF for the test set
					echo "   Generating MLF ..."
					mkdir -p $work_dir/vtln
					nice -n 19 cp $conf $work_dir/vtln/config_score_test.sh
					nice -n 19 cp $conf_global $work_dir/vtln
					echo "do_score_tr=0" >> $work_dir/vtln/config_score_test.sh
					echo "score_with_bigram=0" >> $work_dir/vtln/config_score_test.sh
					chmod ug+x $work_dir/vtln/config_score_test.sh
					${script_dir}/score_pfiles.sh $work_dir/vtln/config_score_test.sh $work_dir
					mv $out_dir/results.txt $out_dir/results_b.txt
					echo "Results without VTLN on the test set!!!" > $out_dir/results.txt
					cat $out_dir/results_b.txt >> $out_dir/results.txt
					rm $out_dir/results_b.txt

					${script_dir}/misc/extractMLFLevel.pl $out_dir/rec_m.mlf 0 | sed 's/\.rec/\.lab/' > $out_dir/vtln/test_states.mlf
				else
					echo "Using recognized labels from $pfile_dir..."
					if [ ! -e $out_dir/vtln/test_states.mlf ] ; then
						nice -n 19 cp $pfile_dir/test_states.mlf $out_dir/vtln/test_states.mlf
					fi
				fi

				# create "waveform - mel banks" list
				if [ ! -e $pfile_dir/configs ] ; then
					echo "ERROR: file '$pfile_dir/configs' not found!"
					exit 1
				fi

				data_dir_test=`grep data_dir_test $pfile_dir/configs | tr '=' ' ' | awk '{print $2}'`
				data_file_suffix=`grep data_file_suffix $pfile_dir/configs | tr '=' ' ' | awk '{print $2}'`

				mkdir -p $out_dir/vtln/features/

				find $data_dir_test -name "*.$data_file_suffix" > $work_dir/vtln/test_dir_files.scp

				awk '{print $1 ".'$data_file_suffix' '$out_dir/vtln/features/'" $1 ".mb"} ' $test_list > $work_dir/vtln/test_wav_mb1.scp

				${script_dir}/misc/expand_paths.pl $work_dir/vtln/test_wav_mb1.scp $work_dir/vtln/test_dir_files.scp > $work_dir/vtln/test_wav_mb.scp


				# vtln
				echo "   Searching for the best warping factors..."

				mkdir -p $out_dir/vtln/log
				${script_dir}/pvtln.pl ${script_dir} $vtln_ncpu $conf $out_dir/vtln $work_dir/vtln/test_wav_mb.scp $pfile_dir/hcopy.cfg $out_dir/vtln/test_states.mlf $out_dir/vtln/log/test.log $out_dir/vtln/log

				awk '{print $2}' $work_dir/vtln/test_wav_mb.scp > $work_dir/vtln/test_mb.scp
				echo "   Generating pfile ..."
				mkdir -p $work_dir/vtln/pfiles
				${bin_dir}/feacat -i $work_dir/vtln/test_mb.scp -l -ip htk -o $work_dir/vtln/pfiles/test_nolab.pfile
				find $out_dir/vtln/features -name "*.mb" -exec rm {} \;

				# create numerical labels and add them into the pfile
				nlabs=`pfile_info $test_tnn | sed '1 d' | awk '{print $5}'`
				if [ "$nlabs" != "0" ] ; then
					echo "   Adding reference labels ..."
					${bin_dir}/pfile_labs -i $test_tnn -f $test_list -d $pfile_dir/dicts/dict -sl $work_dir/vtln/test_ref.mlf -w
					${bin_dir}/pfile_labs -i $work_dir/vtln/pfiles/test_nolab.pfile -o $work_dir/vtln/pfiles/test.pfile -f $test_list -d $pfile_dir/dicts/dict -al $work_dir/vtln/test_ref.mlf
					rm -f $work_dir/vtln/pfiles/test_nolab.pfile
				else
					mv $work_dir/vtln/pfiles/test_nolab.pfile $work_dir/vtln/pfiles/test.pfile
				fi

				if [ ! $out_dir/vtln/pfiles/test.pfile -ef $out_dir/vtln/pfiles/test.pfile ] ; then
					mkdir -p $out_dir/vtln/pfiles
					cp $work_dir/vtln/pfiles/test.pfile $out_dir/vtln/pfiles/test.pfile
				fi
			fi
		fi

		if [ "$do_te_vtln_fwd" = "1" ] ; then

			ex_vtln_test=0
			if [ -e $work_dir/ffout/test_novtln.pfile ] || [ -e $out_dir/ffout/test_novtln.pfile ] ; then
				ex_vtln_test=1
			fi

			ex_test=0
			if [ -e $work_dir/ffout/test.pfile ] || [ -e $out_dir/ffout/test.pfile ] ; then
				ex_test=1
			fi

			if [ $ex_vtln_test = 0 ] || ( [ $ex_vtln_test = 1 ] && [ $ex_test = 0 ] ) ; then

				if [ ! -e $work_dir/ffout/test_novtln.pfile ] ; then
					mv $work_dir/ffout/test.pfile $work_dir/ffout/test_novtln.pfile
				fi

				mkdir -p $out_dir/vtln/pfiles
				${script_dir}/streams_forward_pass.sh $conf test_vtln $out_dir/vtln/pfiles/test.pfile $work_dir/ffout/test.pfile

				if [ $keep_test_merged_band_posteriors == 1 ] ; then
					mkdir -p $out_dir/pfiles
					if [ ! $work_dir/pfiles/test_mnn.pfile -ef $out_dir/pfiles/test_mnn.pfile ] ; then
						rm -f $out_dir/pfiles/test_mnn.pfile
						mv $work_dir/pfiles/test_mnn.pfile $out_dir/pfiles/test_mnn.pfile
					fi
				else
					rm -f $work_dir/pfiles/test_mnn.pfile
				fi

				rm -f $work_dir/pfiles/test_mnn_postpr.pfile
			fi
		fi

	fi
fi

# scoring -----------------------------------------------------------------------------------------------
if [ "$do_scoring" = "1" ] ; then
	echo "Scoring..."
	if [ ! -e $out_dir/results.txt ] ; then
		if [ ! -e $work_dir/ffout/test.pfile ] ; then
			nice -n 19 cp $out_dir/ffout/test.pfile $work_dir/ffout/test.pfile
		fi
		if [ ! -e $work_dir/ffout/cv.pfile ] ; then
			nice -n 19 cp $out_dir/ffout/cv.pfile $work_dir/ffout/cv.pfile
		fi
		if [ ! -e $work_dir/ffout/train.pfile ] && [ "$do_score_tr" = "1" ] ; then
			nice -n 19 cp $out_dir/ffout/train.pfile $work_dir/ffout/train.pfile
		fi

		${script_dir}/score_pfiles.sh $conf $work_dir
	fi
fi

# clean temporal directory ------------------------------------------------------------------------------
# save test posteriors
if [ $keep_test_posteriors == 1 ] && [ -e $work_dir/ffout/test.pfile ] ; then
	mkdir -p $out_dir/ffout
	if [ ! $out_dir/ffout/test.pfile -ef $work_dir/ffout/test.pfile ] ; then
		rm -f $out_dir/ffout/test.pfile
		mv $work_dir/ffout/test.pfile $out_dir/ffout/test.pfile
	fi
else
	rm -f $work_dir/ffout/test.pfile
fi

if [ $keep_test_posteriors == 1 ] && [ -e $work_dir/ffout/test_novtln.pfile ] ; then
	mkdir -p $out_dir/ffout
	if [ ! $out_dir/ffout/test_novtln.pfile -ef $work_dir/ffout/test_novtln.pfile ] ; then
		rm -f $out_dir/ffout/test_novtln.pfile
		mv $work_dir/ffout/test_novtln.pfile $out_dir/ffout/test_novtln.pfile
	fi
else
	rm -f $work_dir/ffout/test_novtln.pfile
fi

# save cv posteriors
if [ $keep_cv_posteriors == 1 ] && [ -e $work_dir/ffout/cv.pfile ] ; then
	mkdir -p $out_dir/ffout
	if [ ! $out_dir/ffout/cv.pfile -ef $work_dir/ffout/cv.pfile ] ; then
		rm -f $out_dir/ffout/cv.pfile
		mv $work_dir/ffout/cv.pfile $out_dir/ffout/cv.pfile
	fi
else
	rm -f $work_dir/ffout/cv.pfile
fi

# save train posteriors
if [ $keep_train_posteriors == 1 ] && [ -e $work_dir/ffout/train.pfile ] ; then
	mkdir -p $out_dir/ffout
	if [ ! $out_dir/ffout/train.pfile -ef $work_dir/ffout/train.pfile ] ; then
		rm -f $out_dir/ffout/train.pfile
		mv $work_dir/ffout/train.pfile $out_dir/ffout/train.pfile
	fi
else
	rm -f $work_dir/ffout/train.pfile
fi

# prepare new iteraction ---------------------------------------------------------------------------------

if [ "$do_prepare_new_it" = "1" ] ; then
	echo "Preparing new iteration..."

	# get iteration number from the experiment name "xxxxx/it2"
	act_iteration=`expr $exp_dir : ".*/it\([0-9]*\)"`
		
	#  new variables for the config file
	if [ "$act_iteration" = "" ] ; then
		new_exp_dir=$exp_dir/it2
		new_out_dir=$out_dir/it2
		act_iteration=2
	else
		# exp_dir=.../it2
		# -> new_exp_dir=.../it3
		new_iteration=$((act_iteration + 1));
		new_exp_dir=`expr $exp_dir : "\(.*/it\)[0-9]*"`$new_iteration
		new_out_dir=`expr $out_dir : "\(.*/it\)[0-9]*"`$new_iteration
	fi

	# generate new config file
	mkdir -p $new_out_dir/scripts

	if [ "$source_pfile_dir" == "" ] ; then
		new_source_pfile_dir=$pfile_dir
	else
		new_source_pfile_dir=$source_pfile_dir
	fi

	new_source_ffout_dir=$out_dir/ffout
	new_realign_labels=1

	if [ "$vtln_schedule" = "1" ] ; then
		do_tr_vtln=1
		do_te_vtln=1
	fi

	awk '/^exp_dir=/          {print gensub("^exp_dir=\"(.+)\"$", "exp_dir=\"'$new_exp_dir'\"", "");}
             /^source_pfile_dir=/ {print gensub("^source_pfile_dir=(.+)$", "source_pfile_dir='$new_source_pfile_dir'", "");}
             /^source_ffout_dir=/ {print gensub("^source_ffout_dir=(.+)$", "source_ffout_dir='$new_source_ffout_dir'", "");}
             /^realign_labels=/   {print gensub("^realign_labels=(.+)$", "realign_labels='$new_realign_labels'", "");}
             /^do_tr_vtln=/       {print gensub("^do_tr_vtln=(.+)$", "do_tr_vtln='$do_tr_vtln'", "");}
             /^do_te_vtln=/       {print gensub("^do_te_vtln=(.+)$", "do_te_vtln='$do_te_vtln'", "");}
             !/^exp_dir=/ && !/^source_pfile_dir=/ && !/^source_ffout_dir=/ && !/^realign_labels=/ && !/^do_tr_vtln=/ && !/^do_te_vtln=/ {print;}' $conf > $new_out_dir/scripts/run_it.sh

	# - add missing variables
	if [ "`grep "source_pfile_dir=" $new_out_dir/scripts/run_it.sh`" = "" ] ; then
		echo "source_pfile_dir=$new_source_pfile_dir" >> $new_out_dir/scripts/run_it.sh
	fi
	if [ "`grep "source_ffout_dir=" $new_out_dir/scripts/run_it.sh`" = "" ] ; then
		echo "source_ffout_dir=$new_source_ffout_dir" >> $new_out_dir/scripts/run_it.sh
	fi
	if [ "`grep "realign_labels=" $new_out_dir/scripts/run_it.sh`" = "" ] ; then
		echo "realign_labels=$new_realign_labels" >> $new_out_dir/scripts/run_it.sh
	fi
	if [ "`grep "exp_dir=" $new_out_dir/scripts/run_it.sh`" = "" ] ; then
		echo "exp_dir=$new_exp_dir" >> $new_out_dir/scripts/run_it.sh
	fi
	if [ "`grep "do_tr_vtln=" $new_out_dir/scripts/run_it.sh`" = "" ] ; then
		echo "do_tr_vtln=$do_tr_vtln" >> $new_out_dir/scripts/run_it.sh
	fi
	if [ "`grep "do_te_vtln=" $new_out_dir/scripts/run_it.sh`" = "" ] ; then
		echo "do_te_vtln=$do_te_vtln" >> $new_out_dir/scripts/run_it.sh
	fi

	chmod ug+x $new_out_dir/scripts/run_it.sh
	nice -n 19 cp $conf_global $new_out_dir/scripts/_global.sh
	mkdir -p $new_out_dir/sge_log

	if [ "$act_iteration" -le "$max_iterations" ] && [ "$max_iterations" != "1" ] && [ "$do_run_new_it" = "1" ] ; then
	    echo ${script_dir}/all.sh $new_out_dir/scripts/run_it.sh
	    ${script_dir}/all.sh $new_out_dir/scripts/run_it.sh
	fi
fi

# clean output directory ---------------------------------------------------------------------------------
${script_dir}/misc/rm_workdir.sh $conf $work_dir
find $out_dir -type d -empty -exec rmdir {} \; 2> /dev/null
