#!/bin/sh

# This script pass a file trough the neural network

if [ $# -lt 3 ]; then
	echo "USAGE: fwd_FN.sh config_file net_name output_rapbin input_pfile1 [input_pfile2]"
	exit 1
fi

conf=$1
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

net_name=$2

# input pfile
ftr1_file=$4

# target file
nlabs=`${bin_dir}/pfile_info $ftr1_file | tail -n 1 | awk '{ print $5 }'`
if [ $nlabs -ne 0 ] ; then
	hardtarget_file=$ftr1_file
fi

# input pfile - second stream
ftr2_file=$5

# output file
activation_file=$3

# number of sentences in the pfile
fwd_sent_count=`${bin_dir}/pfile_info $ftr1_file | tail -n 1 | awk '{print $1}'`

# number of features in the first stream
tnn_nfea=`${bin_dir}/pfile_info $ftr1_file | tail -n 1 | awk '{ print $7 }'`
ftr1_ftr_count=$tnn_nfea

# number of consequentive frames taken as the neural net input
ftr1_window_len=$window_width
window_extent=$ftr1_window_len

# norms for the first stream (means and variances)
ftr1_norm_file=$out_dir/norms/${net_name}_1.norms

# use the second stream
if [ "$ftr2_file" != "" ] ; then
	ftr2_ftr_count=$ftr1_ftr_count
	ftr2_window_len=$ftr1_window_len
	ftr2_norm_file=$out_dir/norms/${system}_2.norms
	tnn_nfea_2=`${bin_dir}/pfile_info $ftr2_file | tail -n 1 | awk '{ print $7 }'`
	nfea_sum=$((ftr1_window_len * $tnn_nfea + ftr2_window_len * $tnn_nfea_2))
else
	ftr2_ftr_count=0
	ftr2_window_len=0
	ftr2_file=""
	ftr2_norm_file=""
	nfea_sum=$((ftr1_window_len * $tnn_nfea))
fi

# number of neurons in each neural network layer
mlp3_input_size=$nfea_sum
mlp3_hidden_size=$num_hidden_band
mlp3_output_size=$num_classes

# output nonlinearity
if [ "$tnn_out_nonlinearity" != "" ] ; then
	mlp3_output_type=$tnn_out_nonlinearity
else
	mlp3_output_type=softmax
fi

# weights for the neural net
init_weight_file=$out_dir/weights/${net_name}.weights

# loging file
logf=$out_dir/log/${net_name}_fwd_last.log
mkdir -p $out_dir/log

#create output directory
out_path_act=${activation_file%\/*}
mkdir -p $out_path_act

# dump some info about configuration
echo "FORWARD PASS"
echo "EXP_DIR   : $out_dir"
echo "PFILE     : $ftr1_file $ftr2_file"
echo "RAPBIN    : $activation_file"
echo "TEST_SENT : $fwd_sent_count"
echo "FEATURES  : $ftr1_ftr_count $ftr2_ftr_count"
echo "NN INPUT  : $mlp3_input_size"
echo "NN_HIDDEN : $mlp3_hidden_size"
echo "NN_OUTPUT : $mlp3_output_size"

# run the forward pass
${bin_dir}/qnsfwd \
	ftr1_file=${ftr1_file}                         \
	ftr2_file=${ftr2_file}                         \
	ftr1_format=${ftr1_format:=pfile}              \
	ftr2_format=${ftr2_format:=pfile}              \
	ftr1_norm_file=${ftr1_norm_file}               \
	ftr2_norm_file=${ftr2_norm_file}               \
        ftr1_ftr_start=${ftr1_ftr_start:=0}            \
        ftr2_ftr_start=${ftr2_ftr_start:=0}            \
        ftr1_ftr_count=${ftr1_ftr_count:=0}            \
        ftr2_ftr_count=${ftr2_ftr_count:=0}            \
        ftr1_window_offset=${ftr1_window_offset:=0}    \
        ftr2_window_offset=${ftr2_window_offset:=0}    \
        ftr1_window_len=${ftr1_window_len:=0}          \
        ftr2_window_len=${ftr2_window_len:=0}          \
        ftr1_delta_order=${ftr1_delta_order:=0}        \
        ftr2_delta_order=${ftr2_delta_order:=0}        \
        ftr1_delta_win=${ftr1_delta_win:=9}            \
        ftr2_delta_win=${ftr2_delta_win:=9}            \
        unary_window_offset=${unary_window_offset:=0}  \
        hardtarget_file=${hardtarget_file:=""}         \
        hardtarget_format=${hardtarget_format:=pfile}  \
        hardtarget_window_offset=${hardtarget_window_offset:=0} \
        activation_file=${activation_file:=out.ff.rap} \
        activation_format=${activation_format:=pfile}  \
        window_extent=${window_extent:=0}              \
        fwd_sent_start=${fwd_sent_start:=0}            \
        fwd_sent_count=${fwd_sent_count:=0}            \
	init_weight_file=${init_weight_file}           \
        mlp3_input_size=${mlp3_input_size}             \
        mlp3_hidden_size=${mlp3_hidden_size:=300}      \
        mlp3_output_size=${mlp3_output_size}           \
	mlp3_output_type=${mlp3_output_type:=softmax}  \
	mlp3_fx=${mlp3_fx:=false}                      \
	debug=${debug:=0}                              \
	verbose=${verbose:=true}                       \
        cpu="host" >& $logf


more $logf | grep "Recognition accuracy"
cat $out_dir/log/${net_name}_fwd_last.log >> $out_dir/log/${net_name}_fwd.log
rm -f $out_dir/log/${net_name}_fwd_last.log

