#!/usr/bin/perl

# This scripts converts MLF with state alignment into MLF with phoneme alignment
#
# state labels look like lab1__2, lab1__3, lab1__4 ...


if($ARGV[0] eq "")
{
	print "SYNTAX: mlf_st2phn.pl mlf\n";
	exit(1);
}


# process the MLF
open MLF, $ARGV[0] or die "Can not open MLF: $ARGV[0]";

$l = <MLF>;
chomp $l;

die "ERROR: The file does not have the MLF header: $ARGV[0]" if($l ne "#!MLF!#");
print "#!MLF!#\n";

while($file = <MLF>)
{
	$first_start = -1;
	$last_stop = -1;
	$phn = "";
	$prev_phn = "";
	$prev_first_start = -1;
	chomp $file;
	print "$file\n";
	do
	{
		$l = <MLF>;
		chomp $l;
		if($l ne ".")
		{
			($start, $stop, $label) = split /\s+/, $l;
			if($label =~ /(.+)__(.+)/)
			{
				$phn = $1;
				$state = $2;
			}
			else
			{
				$phn = $label;
				$state = 0;
			}
		
			#print "$label $state\n";
				
			if($state == 2 || $state == 0)
			{
				#print "* $state $start\n";
				$first_start = $start;
			}

			$last_stop = $stop;
		}
		else
		{
			$phn = ".";
		}
		
		#print "** $phn\n";
		
		if(($phn ne $prev_phn || $phn eq ".") && $prev_phn ne "")
		{
			$stop = ($phn eq "." ? $last_stop : $start);
			print "$prev_first_start $stop $prev_phn\n";
		}
		
		$prev_phn = $phn;
		$prev_first_start = $first_start;
	} while($l ne ".");
	print ".\n";
}

close MLF;



