#!/bin/bash

conf=$1
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

if [ -e "$conf_global" ] ; then
	source $conf_global
fi
if [ -e "$conf" ] ; then
	source $conf
fi

args="$*"
args=${args#* }     # omit first argument

if [[ "$args" == *learnrate_schedule=list* ]] || [ "$ts_use_quicknet" == "1" ]; then
    ${bin_dir}/qnstrn $args
    exit 0
fi


# find arguments of parameters in the command line
schedule=`expr "$args " : '.*learnrate_schedule=\([^ ]*\) .*'`
schedule_var=`expr "$args " : '.*\(learnrate_schedule=[^ ]* \).*'`

vals=`expr "$args " : '.*learnrate_vals=\([^ ]*\) .*'`
vals_var=`expr "$args " : '.*\(learnrate_vals=[^ ]* \).*'`

init_weight_file=`expr "$args " : '.*init_weight_file=\([^ ]* \).*'`
init_weight_file_var=`expr "$args " : '.*\(init_weight_file=[^ ]* \).*'`

out_weight_file=`expr "$args " : '.*out_weight_file=\([^ ]* \).*'`
out_weight_file_var=`expr "$args " : '.*\(out_weight_file=[^ ]* \).*'`

log_weight_file=`expr "$args " : '.*log_weight_file=\([^ ]* \).*'`
log_weight_file_var=`expr "$args " : '.*\(log_weight_file=[^ ]* \).*'`


# remove parameters from the command line
args=${args/$schedule_var/}
args=${args/$vals_var/}
args=${args/$init_weight_file_var/}
args=${args/$out_weight_file_var/}
args=${args/$log_weight_file_var/}


if [[ $out_weight_file == */* ]] ; then    # if contains "/"
    weight_dir="${out_weight_file%/*}/"
else
    weight_dir=""
fi

weight_name=${out_weight_file##*/}         # cut off path
weight_name=${weight_name%\.*}             # cut off suffix
weight_name=${weight_name% *}              # cut off spaces


orig_out=$out_weight_file
log=${weight_dir}${weight_name}.acc
rm -f $log

if [ -e $orig_out ] ; then
	echo "qnstrn.sh - Output weights '$orig_out' alredy exist..."
	return;
fi

# initial parameters
it=1
ok_it=0
lr="$vals"
prev_cv_acc=-100
prev_train_acc=-100
best_cv_it=1
best_cv_acc=-100
stop="no"
min_it=1
max_it=9999

delta_dec=0.5
delta_stop=0.5
is_dec="no"

# set parameters according configuration values if exist
if [ "$ts_delta_dec" != "" ] ; then
	delta_dec=$ts_delta_dec
fi
if [ "$ts_deta_stop" != "" ] ; then
	delta_stop=$ts_deta_stop
fi
if [ "$ts_min_it" != "" ] ; then
	min_it=$ts_min_it
fi
if [ "$ts_max_it" != "" ] ; then
	max_it=$ts_max_it
fi

# loop - training iterations
while [ "$stop" != "yes" ] ; do

	echo "Starting iteration $it, learning rate $lr..."

	# new arguments
	args1="init_weight_file=$init_weight_file \
		out_weight_file=${weight_dir}${weight_name}.$it.weights \
		log_weight_file=${weight_dir}${weight_name}.$it.log \
		learnrate_schedule=list \
		learnrate_vals=$lr"

	#run one iteration of training with Quicknet
	qn_out=${weight_dir}${weight_name}.${it}.qnout
	if [ ! -e $qn_out ] || [ ! -e ${weight_dir}${weight_name}.$it.weights ] ; then
		${bin_dir}/qnstrn $args $args1 > $qn_out
		rm -f ${weight_dir}${weight_name}.$it.fwd
	fi

	#check if the result is correct
	check=`cat $qn_out | grep "Best CV" | awk '{print $1}'`
	if [ "$check" != "Best" ] ; then
		echo Error during neural network training > /dev/stderr
		exit 1
	fi

	# new arguments for forward pass
	mkdir -p /tmp/$USER
	args2="init_weight_file=${weight_dir}${weight_name}.$it.weights \
	activation_file=/tmp/$USER/$$.pfile"
    
	# run forward pass
	if [ ! -e ${weight_dir}${weight_name}.$it.fwd ] ; then
		${script_dir}/qnstrn_fwd.sh $conf ${weight_dir}${weight_name}.$it.fwd $args $args2
		rm -f /tmp/$$.pfile
	fi

	#check if the result is correct
	check=`grep "Recognition acc" ${weight_dir}${weight_name}.$it.fwd | awk '{print $1}'`
	if [ "$check" != "Recognition" ] ; then
		echo Error during neural network forward pass > /dev/stderr
		exit 1
	fi

	cv_acc=`cat $qn_out | grep "Best CV" | awk '{print $4}' | tr -d '%'`
	train_acc=`grep "Recognition acc" ${weight_dir}${weight_name}.$it.fwd | awk '{print $8}' | tr -d '%'`

	#schedule new iteration
	ret=`echo "$lr $is_dec $train_acc $prev_train_acc $cv_acc $prev_cv_acc $delta_dec $delta_stop $it $min_it $max_it" | awk '
	     {
		lr=$1;
		is_dec=$2;
		train_acc=$3;
		prev_train_acc=$4;
		cv_acc=$5;
		prev_cv_acc=$6;
		delta_dec=$7;
		delta_stop=$8;
		it=$9;
		min_it=$10;
		max_it=$11;
		ret_weights="no";
		stop="no";

		if(train_acc - prev_train_acc < delta_dec)
			is_dec="yes";

		if(train_acc - prev_train_acc < 0)
			ret_weights="yes";

		if(is_dec == "yes")
			lr = lr / 2;

		if(cv_acc - prev_cv_acc < delta_stop && ret_weights != "yes" && it >= min_it)
			stop = "yes";

		if(it >= max_it && ret_weights != "yes")
			stop="yes";

		print stop " " ret_weights " " lr " " delta_dec;
	    }'`

	stop=`echo $ret | awk '{print $1}'`
	ret_weights=`echo $ret | awk '{print $2}'`
	old_lr=$lr
	lr=`echo $ret | awk '{print $3}'`

	echo ">>>>>>>>>>>> |$ret|$train_acc|$prev_train_acc|"

	# find the best cv result
	ret=`echo "$cv_acc $best_cv_acc $best_cv_it $it" | awk '
	    {
		cv_acc=$1;
		best_cv_acc=$2;
		best_cv_it=$3;
		it=$4;
		if(cv_acc > best_cv_acc)
		{
		    best_cv_acc=cv_acc;
		    best_cv_it=it;
		}
		print best_cv_acc " " best_cv_it;
	    }'`
	best_cv_acc=`echo $ret | awk '{print $1}'`
	best_cv_it=`echo $ret | awk '{print $2}'`

	# log text
	if [ $ret_weights == "yes" ] ; then
		cmdd="ret$ok_it"
	else
		ok_it=$it
		cmdd="next$ok_it"
		prev_cv_acc=$cv_acc
		prev_train_acc=$train_acc
	fi

	if [ "$stop" == "yes" ] ; then
		cmdd="stop"
	fi

	#read command file
	if [ -e "${weight_dir}${weight_name}.$it.cmd" ] ; then
		var=`cat "${weight_dir}${weight_name}.$it.cmd"`
		if [ $var == "stop" ] ; then
			stop="yes"
			cmdd="stop"
		fi
		ok_it=`echo $var | awk '{print $1}'`
		lr=`echo $var | awk '{print $2}'`
		cmdd="jump$ok_it"
		ok_qn_out=${weight_dir}${weight_name}.${ok_it}.qnout
        	prev_cv_acc=`cat $ok_qn_out | grep "Best CV" | awk '{print $4}' | tr -d '%'`
		prev_train_acc=`cat $ok_qn_out | grep "Best train" | awk '{print $4}' | tr -d '%'`
	fi

	if [ "$ok_it" == "0" ] ; then
		init_weight_file=""
	else
		init_weight_file=${weight_dir}${weight_name}.$ok_it.weights
	fi

	echo "$it $old_lr $train_acc $cv_acc $cmdd"
	echo "$it $old_lr $train_acc $cv_acc $cmdd" >> $log

	it=$(( $it + 1))

done

if [ "$ts_take_last" = "1" ] ; then
	best_cv_it=$(( $it - 1 ))
fi

cp "${weight_dir}${weight_name}.${best_cv_it}.weights" $orig_out

