#!/bin/sh

# This script pass a file trough the neural network

if [ $# -lt 3 ]; then
	echo "USAGE: qnstrn_fwd.sh config_file log_file qnstrn_command"
	exit 1
fi


conf=$1
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

log=$2

args="$*"
args=${args#* * }     # omit first two argument

#extract arguments for qnstrn
ftr1_file=`expr "$args " : '.*ftr1_file=\([^ ]*\) .*'`
ftr2_file=`expr "$args " : '.*ftr2_file=\([^ ]*\) .*'`
ftr1_format=`expr "$args " : '.*ftr1_format=\([^ ]*\) .*'`
ftr2_format=`expr "$args " : '.*ftr2_format=\([^ ]*\) .*'`
ftr1_norm_file=`expr "$args " : '.*ftr1_norm_file=\([^ ]*\) .*'`
ftr2_norm_file=`expr "$args " : '.*ftr2_norm_file=\([^ ]*\) .*'`
ftr1_ftr_start=`expr "$args " : '.*ftr1_ftr_start=\([^ ]*\) .*'`
ftr2_ftr_start=`expr "$args " : '.*ftr2_ftr_start=\([^ ]*\) .*'`
ftr1_ftr_count=`expr "$args " : '.*ftr1_ftr_count=\([^ ]*\) .*'`
ftr2_ftr_count=`expr "$args " : '.*ftr2_ftr_count=\([^ ]*\) .*'`
ftr1_window_offset=`expr "$args " : '.*ftr1_window_offset=\([^ ]*\) .*'`
ftr2_window_offset=`expr "$args " : '.*ftr2_window_offset=\([^ ]*\) .*'`
ftr1_window_len=`expr "$args " : '.*ftr1_window_len=\([^ ]*\) .*'`
ftr2_window_len=`expr "$args " : '.*ftr2_window_len=\([^ ]*\) .*'`
ftr1_delta_win=`expr "$args " : '.*ftr1_delta_win=\([^ ]*\) .*'`
ftr2_delta_win=`expr "$args " : '.*ftr2_delta_win=\([^ ]*\) .*'`
ftr1_delta_order=`expr "$args " : '.*ftr1_delta_order=\([^ ]*\) .*'`
ftr2_delta_order=`expr "$args " : '.*ftr2_delta_order=\([^ ]*\) .*'`
unary_window_offset=`expr "$args " : '.*unary_window_offset=\([^ ]*\) .*'`
hardtarget_file=`expr "$args " : '.*hardtarget_file=\([^ ]*\) .*'`
hardtarget_format=`expr "$args " : '.*hardtarget_format=\([^ ]*\) .*'`
hardtarget_window_offset=`expr "$args " : '.*hardtarget_window_offset=\([^ ]*\) .*'`
window_extent=`expr "$args " : '.*window_extent=\([^ ]*\) .*'`
fwd_sent_start=`expr "$args " : '.*train_sent_start=\([^ ]*\) .*'`
fwd_sent_count=`expr "$args " : '.*train_sent_count=\([^ ]*\) .*'`
init_weight_file=`expr "$args " : '.*init_weight_file=\([^ ]*\) .*'`
mlp3_input_size=`expr "$args " : '.*mlp3_input_size=\([^ ]*\) .*'`
mlp3_hidden_size=`expr "$args " : '.*mlp3_hidden_size=\([^ ]*\) .*'`
mlp3_output_size=`expr "$args " : '.*mlp3_output_size=\([^ ]*\) .*'`
mlp3_output_type=`expr "$args " : '.*mlp3_output_type=\([^ ]*\) .*'`
mlp3_fx=`expr "$args " : '.*mlp3_fx=\([^ ]*\) .*'`
debug=`expr "$args " : '.*debug=\([^ ]*\) .*'`
verbose=`expr "$args " : '.*verbose=\([^ ]*\) .*'`
cpu=`expr "$args " : '.*cpu=\([^ ]*\) .*'`
activation_file=`expr "$args " : '.*activation_file=\([^ ]*\) .*'`


# run the forward pass
${bin_dir}/qnsfwd \
	ftr1_file=${ftr1_file}                         \
	ftr2_file=${ftr2_file}                         \
	ftr1_format=${ftr1_format:=pfile}              \
	ftr2_format=${ftr2_format:=pfile}              \
	ftr1_norm_file=${ftr1_norm_file}               \
	ftr2_norm_file=${ftr2_norm_file}               \
        ftr1_ftr_start=${ftr1_ftr_start:=0}            \
        ftr2_ftr_start=${ftr2_ftr_start:=0}            \
        ftr1_ftr_count=${ftr1_ftr_count:=0}            \
        ftr2_ftr_count=${ftr2_ftr_count:=0}            \
        ftr1_window_offset=${ftr1_window_offset:=0}    \
        ftr2_window_offset=${ftr2_window_offset:=0}    \
        ftr1_window_len=${ftr1_window_len:=0}          \
        ftr2_window_len=${ftr2_window_len:=0}          \
        ftr1_delta_order=${ftr1_delta_order:=0}        \
        ftr2_delta_order=${ftr2_delta_order:=0}        \
        ftr1_delta_win=${ftr1_delta_win:=9}            \
        ftr2_delta_win=${ftr2_delta_win:=9}            \
        unary_window_offset=${unary_window_offset:=0}  \
        hardtarget_file=${hardtarget_file:=""}         \
        hardtarget_format=${hardtarget_format:=pfile}  \
        hardtarget_window_offset=${hardtarget_window_offset:=0} \
        activation_file=${activation_file:=out.ff.rap} \
        activation_format=${activation_format:=pfile}  \
        window_extent=${window_extent:=0}              \
        fwd_sent_start=${fwd_sent_start:=0}            \
        fwd_sent_count=${fwd_sent_count:=0}            \
	init_weight_file=${init_weight_file}           \
        mlp3_input_size=${mlp3_input_size}             \
        mlp3_hidden_size=${mlp3_hidden_size:=300}      \
        mlp3_output_size=${mlp3_output_size}           \
	mlp3_output_type=${mlp3_output_type:=softmax}  \
	mlp3_fx=${mlp3_fx:=false}                      \
	debug=${debug:=0}                              \
	verbose=${verbose:=true}                       \
        cpu="host" >& $log

more $log | grep "Recognition accuracy"
 
