#!/bin/sh

if [ $# -lt 2 ]; then
	echo "Scoring of pfiles with posteriors"
	echo ""
	echo "USAGE: score_pfiles.sh config_file work_directory"
	exit 1
fi

# command line parsing
conf=$1
work_dir=$2

conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

# work with saved posteriors if there are not just generated ones
mkdir -p $work_dir/ffout

if [ ! -e $work_dir/ffout/test.pfile ] && [ -e $out_dir/ffout/test.pfile ] ; then
	ln -s $out_dir/ffout/test.pfile $work_dir/ffout/test.pfile
fi

if [ ! -e $work_dir/ffout/cv.pfile ] && [ -e $out_dir/ffout/cv.pfile ] ; then
	ln -s $out_dir/ffout/cv.pfile $work_dir/ffout/cv.pfile
fi

if [ ! -e $work_dir/ffout/train.pfile ] && [ -e $out_dir/ffout/train.pfile ] ; then
	ln -s $out_dir/ffout/train.pfile $work_dir/ffout/train.pfile
fi

echo $work_dir
echo $out_dir
echo ----------------------
ls $work_dir/ffout


# check if posteriors exist
if [ ! -e $work_dir/ffout/test.pfile ] || [ ! -e $work_dir/ffout/cv.pfile ] ; then
	echo "ERROR: PFILE with posteriors doesn't exit."
	exit
fi

if [ $do_train_fwd == 1 ] && [ ! -e $work_dir/ffout/train.pfile ] ; then
	echo "ERROR: PFILE with posteriors doesn't exist."
	exit
fi

# temporlal directories
mkdir -p $work_dir/scoring/tmp
mkdir -p $work_dir/scoring/lop

# backup results
mkdir -p $out_dir
if [ -e "${out_dir}/results.txt"  ] ; then
	r=0
	while [  -e "${out_dir}/results.old_$r"  ]  ;  do
		r=$(($r+1))
	done
	echo "    results.txt - > results.old_$r"
	mv ${out_dir}/results.txt ${out_dir}/results.old_$r
fi

# list of lop files for SVite
awk '{print "'$work_dir'/scoring/lop/"  $1 ".lop"}' $train_list > $work_dir/scoring/tmp/train_lop.scp
awk '{print "'$work_dir'/scoring/lop/"  $1 ".lop"}' $test_list > $work_dir/scoring/tmp/test_lop.scp
awk '{print "'$work_dir'/scoring/lop/"  $1 ".lop"}' $cv_list > $work_dir/scoring/tmp/cv_lop.scp

# convert pfile (NN output) into lop files (SVite decoder)
if [ -e $out_dir/vars/logposteriors ] || [ "$are_log_posteriors" == "1" ] ; then
	echo "!!! Log posteriors are at the input" >> $work_dir/results.txt
	take_log=""
else
	take_log="-tr log"
fi

# following two lines produce errors "ran out of output filenames" but save everyting correctly
nice -n 19 ${bin_dir}/feacat -i $work_dir/ffout/test.pfile -ip pfile -olist $work_dir/scoring/tmp/test_lop.scp -op htk $take_log >& /dev/null
nice -n 19 ${bin_dir}/feacat -i $work_dir/ffout/cv.pfile -ip pfile -olist $work_dir/scoring/tmp/cv_lop.scp -op htk $take_log >& /dev/null
if [ "$do_score_tr" = "1" ] ; then
	${bin_dir}/feacat -i $work_dir/ffout/train.pfile -ip pfile -olist $work_dir/scoring/tmp/train_lop.scp -op htk $take_log >& /dev/null
fi

# run SVite and calculate results
${script_dir}/scoring.sh $conf $work_dir/scoring/tmp/train_lop.scp $work_dir/scoring/tmp/test_lop.scp $work_dir/scoring/tmp/cv_lop.scp $ref_mlf $dict $work_dir

if [ "$work_dir" != "$out_dir" ] ; then
	rm -f $out_dir/cv.mlf
	rm -f $out_dir/rec.mlf
	rm -f $out_dir/train_rec.mlf
	rm -f $out_dir/cv_lm.mlf
	rm -f $out_dir/rec_lm.mlf
	rm -f $out_dir/train_rec_lm.mlf
	rm -f $out_dir/results.txt
	cp $work_dir/cv.mlf $out_dir/cv.mlf
	cp $work_dir/rec.mlf $out_dir/rec.mlf
	if [ -e $work_dir/train_rec.mlf ] ; then
		cp $work_dir/train_rec.mlf $out_dir/train_rec.mlf
	fi
	if [ -e $work_dir/cv_lm.mlf ] ; then
		cp $work_dir/cv_lm.mlf $out_dir/cv_lm.mlf
	fi
	if [ -e $work_dir/rec_lm.mlf ] ; then
		cp $work_dir/rec_lm.mlf $out_dir/rec_lm.mlf
	fi
	if [ -e $work_dir/train_rec_lm.mlf ] ; then
		cp $work_dir/train_rec_lm.mlf $out_dir/train_rec_lm.mlf
	fi
	if [ -e $work_dir/train_rec_m.mlf ] ; then
		cp $work_dir/train_rec_m.mlf $out_dir/train_rec_m.mlf
	fi
	if [ -e $work_dir/train_rec_lm_m.mlf ] ; then
		cp $work_dir/train_rec_lm_m.mlf $out_dir/train_rec_lm_m.mlf
	fi
	if [ -e $work_dir/rec_m.mlf ] ; then
		cp $work_dir/rec_m.mlf $out_dir/rec_m.mlf
	fi
	if [ -e $work_dir/rec_lm_m.mlf ] ; then
		cp $work_dir/rec_lm_m.mlf $out_dir/rec_lm_m.mlf
	fi

	cp $work_dir/results.txt $out_dir/results.txt
fi

if [ "$keep_lop" = "1" ] ; then
	mkdir -o $out_dir/lop
	mv -f -r $work_dir/scoring/lop $out_dir/lop
else
	rm -f -r $work_dir/scoring/lop
fi

rm -f -r $work_dir/scoring/tmp
rm -f $work_dir/scoring/*
