#!/bin/sh

# This script trains the trap merger

if [ $# -lt 3 ]; then
	echo "USAGE: traps_train_merger.sh config_file net_name input_pfile"
	exit 1
fi

conf=$1
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

net_name=$2

# number of sentences in the training part
train_sent_count=$mnn_train_sents

# number of sentences in the crossvalidation part
cv_sent_count=$mnn_cv_sents

# start of the crosvalidation part
cv_sent_start=$train_sent_count

# get number of features from the pfile
mnn_nfea=`${bin_dir}/pfile_info $3 | tail -n 1 | awk '{ print $7 }'`

# number of features in the first stream
ftr1_ftr_count=$mnn_nfea

# number of consequentive frames taken as the neural net input
ftr1_window_len=$mnn_window_width
window_extent=$ftr1_window_len

# output file with neural net weights
out_weight_file=$out_dir/weights/${net_name}.weights
log_weight_file=$out_dir/weights/${net_name}.log_weights

# loging file
logf=$out_dir/log/${net_name}.log

# pfile for the first stream
ftr1_file=$3

# norms for the first stream (means and variances)
ftr1_norm_file=$out_dir/norms/${net_name}_1.norms

# use the second stream
if [ "$4" != "" ] ; then
	# get number of features from the pfile
	mnn_nfea_2=`{$bin_dir}/pfile_info $4 | tail -n 1 | awk '{ print $7 }'`
	#
	ftr2_ftr_count=$ftr1_ftr_count
	ftr2_window_len=$ftr1_window_len
	ftr2_file=$4
	ftr2_norm_file=$out_dir/norms/${net_name}_2.norms
	nfea_sum=$((ftr1_window_len * mnn_nfea + ftr2_window_len * mnn_nfea_2))
else
	ftr2_ftr_count=0
	ftr2_window_len=0
	ftr2_file=""
	ftr2_norm_file=""
	nfea_sum=$((ftr1_window_len * mnn_nfea))
fi

# number of neurons in each neural network layer
mlp3_input_size=$nfea_sum
mlp3_hidden_size=$num_hidden_merger
mlp3_output_size=$mnn_num_classes

# output nonlinearity
if [ "$mnn_out_nonlinearity" != "" ] ; then
        mlp3_output_type=$mnn_out_nonlinearity
else
        mlp3_output_type=softmax
fi

mkdir -p $out_dir/log
mkdir -p $out_dir/norms
mkdir -p $out_dir/weights

echo "NEURAL NETWORK TRAINING"
echo "EXP_DIR   : $out_dir"
echo "PFILE     : $ftr1_file $ftr2_file"
echo "TR_SENT   : $train_sent_count"
echo "CV_SENT   : $cv_sent_count"
echo "FEATURES  : $ftr1_ftr_count"
echo "NN INPUT  : $mlp3_input_size"
echo "NN_HIDDEN : $mlp3_hidden_size"
echo "NN_OUTPUT : $mlp3_output_size"

# normalize

${bin_dir}/qnnorm \
	norm_ftrfile=$ftr1_file \
	output_normfile=${ftr1_norm_file:=testdata.norm} \
	first_sent=${first_sent:=0} \
	num_sents=${train_sent_count:=0} \
	debug=${debug:=0} \
	verbose=${verbose:=true} > $logf

# weight normalization coefficients by a window

if [ "$merger_norms_wintype" != "none" ] && [ "$merger_norms_wintype" != "" ] ; then
    mv $ftr1_norm_file ${ftr1_norm_file}.not_norm
    ${script_dir}/misc/weight_norms.pl ${ftr1_norm_file}.not_norm $merger_norms_wintype $merger_norms_nwins > ${ftr1_norm_file}
fi	
	
if [ "$4" != "" ] ; then
	${bin_dir} \
		qnnorm	norm_ftrfile=${ftr2_file} \
		output_normfile=${ftr2_norm_file:=testdata.norm} \
		first_sent=${first_sent:=0} \
		num_sents=${train_sent_count:=0} \
	        debug=${debug:=0} \
		verbose=${verbose:=true} >> $logf
	
	if [ "$merger_norms_wintype" != "none" ] && [ "$merger_norms_wintype" != "" ] ; then
		mv $ftr2_norm_file ${ftr2_norm_file}.not_norm
		${script_dir}/misc/weight_norms.pl ${ftr2_norm_file}.not_norm $merger_norms_wintype $merger_norms_nwins > ${ftr2_norm_file}
	fi
fi

# train the net
${script_dir}/qnstrn.sh $conf \
	ftr1_file=${ftr1_file}            \
	ftr2_file=${ftr2_file}            \
	ftr1_format=${ftr1_format:=pfile} \
	ftr2_format=${ftr2_format:=pfile} \
	ftr1_norm_file=${ftr1_norm_file}  \
	ftr2_norm_file=${ftr2_norm_file}  \
        ftr1_ftr_start=${ftr1_ftr_start:=0} \
        ftr2_ftr_start=${ftr2_ftr_start:=0} \
        ftr1_ftr_count=${ftr1_ftr_count:=0} \
        ftr2_ftr_count=${ftr2_ftr_count:=0} \
        ftr1_window_offset=${ftr1_window_offset:=0}    \
        ftr2_window_offset=${ftr2_window_offset:=0}    \
        ftr1_window_len=${ftr1_window_len:=0}          \
        ftr2_window_len=${ftr2_window_len:=0}          \
	ftr1_delta_win=${ftr1_delta_win:=9}            \
	ftr2_delta_win=${ftr2_delta_win:=9}            \
	ftr1_delta_order=${ftr1_delta_order:=0}        \
	ftr2_delta_order=${ftr2_delta_order:=0}        \
        unary_window_offset=${unary_window_offset:=0}  \
        hardtarget_file=${hardtarget_file:=$ftr1_file} \
        hardtarget_format=${hardtarget_format:=pfile}  \
        hardtarget_window_offset=${hardtarget_window_offset:=0} \
        window_extent=${window_extent:=0}               \
	train_cache_frames=${train_cache_frames:=12000} \
        cv_sent_start=${cv_sent_start:=0}               \
        cv_sent_count=${cv_sent_count}                  \
        train_sent_start=${train_sent_start:=0}         \
        train_sent_count=${train_sent_count}            \
	init_weight_file=${init_weight_file}            \
        mlp3_input_size=${mlp3_input_size}              \
        mlp3_hidden_size=${mlp3_hidden_size:=300}       \
        mlp3_output_size=${mlp3_output_size}            \
	mlp3_output_type=${mlp3_output_type:=softmax}   \
	out_weight_file=${out_weight_file:=out.weights} \
	log_weight_file=${log_weight_file:=log.weights} \
	learnrate_schedule=${learnrate_schedule:=newbob}   \
	learnrate_epochs=${learnrate_epochs:=9999}         \
	init_random_bias_min=${init_random_bias_min:=-4.1} \
	init_random_bias_max=${init_random_bias_max:=-3.9} \
	init_random_weight_min=${init_random_weight_min:=-0.1} \
	init_random_weight_max=${init_random_weight_max:=0.1}  \
	learnrate_vals=${learnrate_vals:=0.008}                \
	mlp3_fx=${mlp3_fx:=false}                              \
	train_cache_seed=${train_cache_seed:=0}                \
	debug=${debug:=0}                                      \
	verbose=${verbose:=true}                               \
        cpu=${cpu:=host} | tee > $logf

