#!/bin/bash

if [ $# -lt 5 ]; then
	echo "VTLN"
	echo ""
	echo "USAGE: vtln.sh config_file wav_mb_list hcopy_conf mlf [log]"
	exit 1
fi

# command line parsing
conf=$1
wav_mb_list=$2
hcopy_conf=$3
mlf=$4
log_file=$5

if [ "$log_file" = "" ] ; then
	log_file="/dev/null"
fi

# source config files
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

#echo "
#CONF:     $conf
#WORK_DIR: $work_dir
#IN:       $wav_list
#LAB:      $num_labels
#OUT:      $output_pfile
#REF LAB:  $ref_labels"

# allocate temporal directory
work_dir1=`${script_dir}/misc/get_workdir.sh $conf`
echo "Working directory for realignment (all.sh): $work_dir1"
machine=`uname -n`
echo "Computer:                     $machine"

list_name="${wav_mb_list##*/}"
work_dir=$work_dir1/vtln/tmps/${list_name}.tmp
mkdir -p $work_dir

nsent=`wc $wav_mb_list | awk '{print $1}'`

rm -f $log_file

# split MLF
mkdir -p $work_dir/labels

awk '{print $1}' $wav_mb_list | tr '/' ' ' | awk '{ gsub("\\..*", ""); print "'$work_dir'/labels/" $NF ".lab"}' > $work_dir/labels.lst
touch $work_dir/empty
#mlf1=$tmp/mlf_flt.mlf
#${bin_dir}/HLEd -I $mlf -i $mlf1 -S $tmp/labels.lst $tmp/empty
${bin_dir}/HLEd -I $mlf -l $work_dir/labels -S $work_dir/labels.lst $work_dir/empty

for file in `cat $wav_mb_list | tr ' ' '|'` ; do

	file1=`echo $file | tr '|' ' ' | awk '{print $1}'`
	file2=`echo $file | tr '|' ' ' | awk '{print $2}'`

	echo $file1

	file_name=${file##*/}
	file_name=${file_name%\.*}
	label_file="$work_dir/labels/$file_name.lab"

	out=`${script_dir}/vtln_1file.sh $conf $work_dir $file1 $file2 $label_file $hcopy_conf`
	echo "$out"
	wf=`echo $out | tr ' ' '\n' | tail -n 1`
	echo $file1 $wf >> $log_file
done

# clean temporal directory
rm -f -r $work_dir
${script_dir}/misc/rm_workdir.sh $conf $work_dir1
